
/* GRJ, Apr/May 2020
 */


package beast.evolution.operators;

import beast.core.Operator;
import beast.evolution.tree.NodePoMo;
import beast.evolution.tree.TreePoMo;
import beast.util.Randomizer;

public class PoMoCoVCountsWalker extends TreeOperatorTransmission {


    @Override
    public void initAndValidate() {

    }

    @Override
    public double proposal() {
        TreePoMo tree = treeInput.get(this);
        if (tree.getUnobservedCases()) {
            double logHR =  unobservedCountsWalk();
            tree.checkTree();
            return logHR;
        } else {
            return Double.NEGATIVE_INFINITY;
        }
    }

    private double unobservedCountsWalk() {
        TreePoMo tree = treeInput.get(this);
        assert tree.getNodeCount() > 1;
        NodePoMo[] nodes = tree.getNodesAsArray();
        NodePoMo node = null;
        do {
            int nodeNr = Randomizer.nextInt(tree.getNodeCount());
            if (!nodes[nodeNr].isRoot()) {
                node = nodes[nodeNr];
            }
        } while (node == null);
        int u = node.getUnobservedCount();
        int du = 2 * Randomizer.nextInt(2) - 1;
        if (u + du < 0) {
            return Double.NEGATIVE_INFINITY;
        }
        node.setUnobservedCount(u + du);
        return 0.0;
    }



}


